@@ -1,16 +1,21 @@
-from typing import Any, Dict, List, Literal, Sequence
+from typing import Any, Dict, List, Literal, Sequence, TypeVar
 
-from grai_client.schemas.schema import Schema
 from grai_schemas import config as base_config
 from grai_schemas.generics import DefaultValue
+from grai_schemas.v1 import SourcedEdgeV1, SourcedNodeV1, SourceV1
 from grai_schemas.v1.metadata.edges import (
     ColumnToColumnMetadata,
-    EdgeTypeLabels,
+    EdgeMetadataTypeLabels,
     GenericEdgeMetadataV1,
     TableToColumnMetadata,
     TableToTableMetadata,
 )
-from grai_schemas.v1.metadata.nodes import ColumnMetadata, NodeTypeLabels, TableMetadata
+from grai_schemas.v1.metadata.nodes import (
+    ColumnMetadata,
+    NodeMetadataTypeLabels,
+    TableMetadata,
+)
+from grai_schemas.v1.source import SourceSpec
 from multimethod import multimethod
 
 from grai_source_postgres.models import (
@@ -19,21 +24,46 @@ from grai_source_postgres.models import (
     Column,
     ColumnConstraint,
     ColumnID,
-    Constraint,
     Edge,
     Table,
     TableID,
 )
 from grai_source_postgres.package_definitions import config
 
+T = TypeVar("T")
+X = TypeVar("X")
+Y = TypeVar("Y")
+
 
 @multimethod
 def build_grai_metadata(current: Any, desired: Any) -> None:
+    """
+
+    Args:
+        current (Any):
+        desired (Any):
+
+    Returns:
+
+    Raises:
+
+    """
     raise NotImplementedError(f"No adapter between {type(current)} and {type(desired)} for value {current}")
 
 
 @build_grai_metadata.register
 def build_grai_metadata_from_column(current: Column, version: Literal["v1"] = "v1") -> ColumnMetadata:
+    """
+
+    Args:
+        current (Column):
+        version (Literal["v1"], optional):  (Default value = "v1")
+
+    Returns:
+
+    Raises:
+
+    """
     default_value = current.default_value
     if current.default_value is not None:
         default_value = DefaultValue(
@@ -44,7 +74,7 @@ def build_grai_metadata_from_column(current: Column, version: Literal["v1"] = "v
 
     data = {
         "version": version,
-        "node_type": NodeTypeLabels.column.value,
+        "node_type": NodeMetadataTypeLabels.column.value,
         "node_attributes": {
             "data_type": current.data_type,
             "default_value": default_value,
@@ -53,6 +83,7 @@ def build_grai_metadata_from_column(current: Column, version: Literal["v1"] = "v
             and current.column_constraint.value == ColumnConstraint.primary_key.value,
             "is_unique": current.column_constraint and current.column_constraint.value in UNIQUE_COLUMN_CONSTRAINTS,
         },
+        "tags": [config.metadata_id],
     }
 
     return ColumnMetadata(**data)
@@ -60,10 +91,22 @@ def build_grai_metadata_from_column(current: Column, version: Literal["v1"] = "v
 
 @build_grai_metadata.register
 def build_grai_metadata_from_node(current: Table, version: Literal["v1"] = "v1") -> TableMetadata:
+    """
+
+    Args:
+        current (Table):
+        version (Literal["v1"], optional):  (Default value = "v1")
+
+    Returns:
+
+    Raises:
+
+    """
     data = {
         "version": version,
-        "node_type": NodeTypeLabels.table.value,
+        "node_type": NodeMetadataTypeLabels.table.value,
         "node_attributes": {},
+        "tags": [config.metadata_id],
     }
 
     return TableMetadata(**data)
@@ -71,28 +114,64 @@ def build_grai_metadata_from_node(current: Table, version: Literal["v1"] = "v1")
 
 @build_grai_metadata.register
 def build_grai_metadata_from_edge(current: Edge, version: Literal["v1"] = "v1") -> GenericEdgeMetadataV1:
-    data = {"version": version}
+    """
+
+    Args:
+        current (Edge):
+        version (Literal["v1"], optional):  (Default value = "v1")
+
+    Returns:
+
+    Raises:
+
+    """
+    data = {"version": version, "tags": [config.metadata_id]}
 
     if isinstance(current.source, TableID):
         if isinstance(current.destination, ColumnID):
-            data["edge_type"] = EdgeTypeLabels.table_to_column.value
+            data["edge_type"] = EdgeMetadataTypeLabels.table_to_column.value
             return TableToColumnMetadata(**data)
         elif isinstance(current.destination, TableID):
-            data["edge_type"] = EdgeTypeLabels.table_to_table.value
+            data["edge_type"] = EdgeMetadataTypeLabels.table_to_table.value
             return TableToTableMetadata(**data)
     elif isinstance(current.source, ColumnID):
         if isinstance(current.destination, ColumnID):
-            data["edge_type"] = EdgeTypeLabels.column_to_column.value
+            data["edge_type"] = EdgeMetadataTypeLabels.column_to_column.value
             return ColumnToColumnMetadata(**data)
 
+    data["edge_type"] = EdgeMetadataTypeLabels.generic.value
+    return GenericEdgeMetadataV1(**data)
+
 
 @multimethod
-def build_postgres_metadata(current: Any, desired: Any) -> None:
+def build_app_metadata(current: Any, desired: Any) -> None:
+    """
+
+    Args:
+        current (Any):
+        desired (Any):
+
+    Returns:
+
+    Raises:
+
+    """
     raise NotImplementedError(f"No adapter between {type(current)} and {type(desired)} for value {current}")
 
 
-@build_postgres_metadata.register
+@build_app_metadata.register
 def build_metadata_from_column(current: Column, version: Literal["v1"] = "v1") -> Dict:
+    """
+
+    Args:
+        current (Column):
+        version (Literal["v1"], optional):  (Default value = "v1")
+
+    Returns:
+
+    Raises:
+
+    """
     data = {
         "table_name": current.table,
         "schema": current.column_schema,
@@ -101,8 +180,19 @@ def build_metadata_from_column(current: Column, version: Literal["v1"] = "v1") -
     return data
 
 
-@build_postgres_metadata.register
+@build_app_metadata.register
 def build_metadata_from_edge(current: Edge, version: Literal["v1"] = "v1") -> Dict:
+    """
+
+    Args:
+        current (Edge):
+        version (Literal["v1"], optional):  (Default value = "v1")
+
+    Returns:
+
+    Raises:
+
+    """
     data = {
         "definition": current.definition,
         "constraint_type": current.constraint_type.name,
@@ -112,8 +202,19 @@ def build_metadata_from_edge(current: Edge, version: Literal["v1"] = "v1") -> Di
     return data
 
 
-@build_postgres_metadata.register
+@build_app_metadata.register
 def build_metadata_from_node(current: Table, version: Literal["v1"] = "v1") -> Dict:
+    """
+
+    Args:
+        current (Table):
+        version (Literal["v1"], optional):  (Default value = "v1")
+
+    Returns:
+
+    Raises:
+
+    """
     data = {
         "schema": current.table_schema,
         "table_type": current.table_type.value,
@@ -123,51 +224,117 @@ def build_metadata_from_node(current: Table, version: Literal["v1"] = "v1") -> D
 
 
 def build_metadata(obj, version):
-    return {
-        base_config.metadata_id: build_grai_metadata(obj, version),
-        config.metadata_id: build_postgres_metadata(obj, version),
-    }
+    """
+
+    Args:
+        obj:
+        version:
+
+    Returns:
+
+    Raises:
+
+    """
+    integration_meta = build_app_metadata(obj, version)
+    integration_meta["grai"] = build_grai_metadata(obj, version)
+
+    return integration_meta
 
 
 @multimethod
 def adapt_to_client(current: Any, desired: Any):
+    """
+
+    Args:
+        current (Any):
+        desired (Any):
+
+    Returns:
+
+    Raises:
+
+    """
     raise NotImplementedError(f"No adapter between {type(current)} and {type(desired)}")
 
 
 @adapt_to_client.register
-def adapt_column_to_client(current: Column, version: Literal["v1"] = "v1"):
+def adapt_column_to_client(current: Column, source: SourceSpec, version: Literal["v1"]) -> SourcedNodeV1:
+    """
+
+    Args:
+        current (Column):
+        version (Literal["v1"], optional):  (Default value = "v1")
+
+    Returns:
+
+    Raises:
+
+    """
     spec_dict = {
         "name": current.full_name,
         "namespace": current.namespace,
         "display_name": current.name,
-        "data_source": config.integration_name,
+        "data_source": source,
         "metadata": build_metadata(current, version),
     }
-    return Schema.to_model(spec_dict, version=version, typing_type="Node")
+    return SourcedNodeV1.from_spec(spec_dict)
 
 
 @adapt_to_client.register
-def adapt_table_to_client(current: Table, version: Literal["v1"] = "v1"):
+def adapt_table_to_client(current: Table, source: SourceSpec, version: Literal["v1"]) -> SourcedNodeV1:
+    """
+
+    Args:
+        current (Table):
+        version (Literal["v1"], optional):  (Default value = "v1")
+
+    Returns:
+
+    Raises:
+
+    """
     spec_dict = {
         "name": current.full_name,
         "namespace": current.namespace,
         "display_name": current.name,
-        "data_source": config.integration_name,
+        "data_source": source,
         "metadata": build_metadata(current, version),
     }
-    return Schema.to_model(spec_dict, version=version, typing_type="Node")
+    return SourcedNodeV1.from_spec(spec_dict)
 
 
 def make_name(node1: ID, node2: ID) -> str:
+    """
+
+    Args:
+        node1 (ID):
+        node2 (ID):
+
+    Returns:
+
+    Raises:
+
+    """
     node1_name = f"{node1.namespace}:{node1.full_name}"
     node2_name = f"{node2.namespace}:{node2.full_name}"
     return f"{node1_name} -> {node2_name}"
 
 
 @adapt_to_client.register
-def adapt_edge_to_client(current: Edge, version: Literal["v1"] = "v1"):
+def adapt_edge_to_client(current: Edge, source: SourceSpec, version: Literal["v1"]) -> SourcedEdgeV1:
+    """
+
+    Args:
+        current (Edge):
+        version (Literal["v1"], optional):  (Default value = "v1")
+
+    Returns:
+
+    Raises:
+
+    """
     spec_dict = {
-        "data_source": config.integration_name,
+        "data_source": source,
         "name": make_name(current.source, current.destination),
         "namespace": current.source.namespace,
         "source": {
@@ -181,9 +348,25 @@ def adapt_edge_to_client(current: Edge, version: Literal["v1"] = "v1"):
         "metadata": build_metadata(current, version),
     }
 
-    return Schema.to_model(spec_dict, version=version, typing_type="Edge")
+    return SourcedEdgeV1.from_spec(spec_dict)
 
 
 @adapt_to_client.register
-def adapt_list_to_client(objs: Sequence, version: Literal["v1"]) -> List:
-    return [adapt_to_client(item, version) for item in objs]
+def adapt_list_to_client(objs: Sequence, source: SourceSpec, version: Literal["v1"]) -> List:
+    """
+
+    Args:
+        objs (Sequence):
+        version (Literal["v1"]):
+
+    Returns:
+
+    Raises:
+
+    """
+    return [adapt_to_client(item, source, version) for item in objs]
+
+
+@adapt_to_client.register
+def adapt_source_spec_v1_to_client(obj: X, source: SourceV1, version: Y) -> T:
+    return adapt_to_client(obj, source.spec, version)
