@@ -1,7 +1,7 @@
 import os
 from functools import cached_property
-from itertools import chain
-from typing import Any, Callable, Dict, List, Optional, Union
+from itertools import chain, tee
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 import psycopg2
 import psycopg2.extras
@@ -17,6 +17,18 @@ from grai_source_postgres.models import (
 
 
 def get_from_env(label: str, default: Optional[Any] = None, validator: Callable = None):
+    """
+
+    Args:
+        label (str):
+        default (Optional[Any], optional):  (Default value = None)
+        validator (Callable, optional):  (Default value = None)
+
+    Returns:
+
+    Raises:
+
+    """
     env_key = f"GRAI_POSTGRES_{label.upper()}"
     result = os.getenv(env_key, default)
     if result is None:
@@ -29,6 +41,8 @@ def get_from_env(label: str, default: Optional[Any] = None, validator: Callable
 
 
 class PostgresConnector:
+    """ """
+
     def __init__(
         self,
         dbname: Optional[str] = None,
@@ -62,11 +76,21 @@ class PostgresConnector:
 
     @property
     def connection_string(self) -> str:
+        """
+
+        Args:
+
+        Returns:
+
+        Raises:
+
+        """
         return (
             f"dbname={self.dbname} host='{self.host}' user='{self.user}' password='{self.password}' port='{self.port}'"
         )
 
     def connect(self):
+        """ """
         if self._connection is None:
             self._connection = psycopg2.connect(self.connection_string)
             self._is_connected = True
@@ -74,16 +98,37 @@ class PostgresConnector:
 
     @property
     def connection(self):
+        """ """
         if self._connection is None:
             raise Exception("Not connected, call `.connect()")
         return self._connection
 
     def close(self) -> None:
+        """
+
+        Args:
+
+        Returns:
+
+        Raises:
+
+        """
         self.connection.close()
         self._connection = None
         self._is_connected = False
 
     def query_runner(self, query: str, param_dict: Dict = {}) -> List[Dict]:
+        """
+
+        Args:
+            query (str):
+            param_dict (Dict, optional):  (Default value = {})
+
+        Returns:
+
+        Raises:
+
+        """
         cursor = self.connection.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
         cursor.execute(query, param_dict)
         result = cursor.fetchall()
@@ -91,10 +136,16 @@ class PostgresConnector:
 
     @cached_property
     def tables(self) -> List[Table]:
-        """
-        Create and return a list of dictionaries with the
+        """Create and return a list of dictionaries with the
         schemas and names of tables in the database
         connected to by the connection argument.
+
+        Args:
+
+        Returns:
+
+        Raises:
+
         """
 
         query = """
@@ -111,9 +162,15 @@ class PostgresConnector:
 
     @cached_property
     def columns(self) -> List[Column]:
-        """
-        Creates and returns a list of dictionaries for the specified
+        """Creates and returns a list of dictionaries for the specified
         schema.table in the database connected to.
+
+        Args:
+
+        Returns:
+
+        Raises:
+
         """
         query = f"""
             SELECT
@@ -144,7 +201,17 @@ class PostgresConnector:
         """
         return [Column(**result, namespace=self.namespace) for result in self.query_runner(query)]
 
-    def get_table_columns(self, table: Table):
+    def get_table_columns(self, table: Table) -> List[Column]:
+        """
+
+        Args:
+            table (Table):
+
+        Returns:
+
+        Raises:
+
+        """
         table_id = (table.table_schema, table.name)
         if table_id in self.column_map:
             return self.column_map[table_id]
@@ -152,8 +219,17 @@ class PostgresConnector:
             raise Exception(f"No columns found for table with schema={table.table_schema} and name={table.name}")
 
     @cached_property
-    def column_map(self):
-        result_map = {}
+    def column_map(self) -> Dict[Tuple[str, str], List[Column]]:
+        """
+
+        Args:
+
+        Returns:
+
+        Raises:
+
+        """
+        result_map: Dict[Tuple[str, str], List[Column]] = {}
         for col in self.columns:
             table_id = (col.column_schema, col.table)
             result_map.setdefault(table_id, [])
@@ -163,12 +239,17 @@ class PostgresConnector:
     @cached_property
     def foreign_keys(self) -> List[Edge]:
         """This needs to be tested / evaluated
-        :param connection:
-        :param table:
-        :return:
+
+        Args:
+
+        Returns:
+
+        Raises:
+
         """
         # query is from https://dba.stackexchange.com/questions/36979/retrieving-all-pk-and-fk/37068#37068
         # Only need constraint_types == 'f' for foreign keys but the others might be useful someday.
+
         query = """
             SELECT c.conname                                         AS constraint_name,
                    c.contype                                         AS constraint_type,
@@ -188,23 +269,56 @@ class PostgresConnector:
                    LEFT JOIN pg_class f_tbl ON f_tbl.oid = c.confrelid
                    LEFT JOIN pg_namespace f_sch ON f_sch.oid = f_tbl.relnamespace
                    LEFT JOIN pg_attribute f_col ON (f_col.attrelid = f_tbl.oid AND f_col.attnum = f_u.attnum)
+            WHERE sch.nspname IN %(schemas)s AND f_sch.nspname IN %(schemas)s
             GROUP BY constraint_name, constraint_type, "self_schema", "self_table", definition, "foreign_schema", "foreign_table"
             ORDER BY "self_schema", "self_table";
         """
+
+        query_params = {"schemas": tuple({table.table_schema for table in self.tables})}
+        results = self.query_runner(query, query_params)
+
         addtl_args = {
             "namespace": self.namespace,
         }
-        results = self.query_runner(query)
         filtered_results = (result for result in results if result["constraint_type"] == "f")
-        return [EdgeQuery(**fk, **addtl_args).to_edge() for fk in filtered_results]
+        result_iter = (EdgeQuery(**fk, **addtl_args).to_edge() for fk in filtered_results if fk is not None)
+        return list(result_iter)
 
     def get_nodes(self) -> List[PostgresNode]:
+        """
+
+        Args:
+
+        Returns:
+
+        Raises:
+
+        """
         return list(chain(self.tables, self.columns))
 
-    def get_edges(self):
-        return list(chain(*[t.get_edges() for t in self.tables], self.foreign_keys))
+    def get_edges(self) -> List[Edge]:
+        """
 
-    def get_nodes_and_edges(self):
+        Args:
+
+        Returns:
+
+        Raises:
+
+        """
+        edge_iter = chain(*[t.get_edges() for t in self.tables], self.foreign_keys)
+        return list(edge_iter)
+
+    def get_nodes_and_edges(self) -> Tuple[List[PostgresNode], List[Edge]]:
+        """
+
+        Args:
+
+        Returns:
+
+        Raises:
+
+        """
         nodes = self.get_nodes()
         edges = self.get_edges()
 
