@@ -5,23 +5,41 @@ from pydantic import BaseModel, Field, root_validator, validator
 
 
 class PostgresNode(BaseModel):
+    """ """
+
     pass
 
 
 class ID(PostgresNode):
+    """ """
+
     name: str
     namespace: str
     full_name: str
 
     class Config:
+        """ """
+
         extra = "forbid"
 
 
 class TableID(ID):
+    """ """
+
     table_schema: str
 
     @root_validator(pre=True)
     def make_full_name(cls, values):
+        """
+
+        Args:
+            values:
+
+        Returns:
+
+        Raises:
+
+        """
         full_name = values.get("full_name", None)
         if values.get("full_name", None) is None:
             values["full_name"] = f"{values['table_schema']}.{values['name']}"
@@ -29,11 +47,23 @@ class TableID(ID):
 
 
 class ColumnID(ID):
+    """ """
+
     table_schema: str
     table_name: str
 
     @root_validator(pre=True)
     def make_full_name(cls, values):
+        """
+
+        Args:
+            values:
+
+        Returns:
+
+        Raises:
+
+        """
         full_name = values.get("full_name", None)
         if values.get("full_name", None) is None:
             values["full_name"] = f"{values['table_schema']}.{values['table_name']}.{values['name']}"
@@ -41,6 +71,8 @@ class ColumnID(ID):
 
 
 class ColumnConstraint(Enum):
+    """ """
+
     primary_key = "p"
     unique = "u"
     foreign_key = "f"
@@ -53,6 +85,8 @@ UNIQUE_COLUMN_CONSTRAINTS = {ColumnConstraint.primary_key.value, ColumnConstrain
 
 
 class Column(PostgresNode):
+    """ """
+
     name: str = Field(alias="column_name")
     table: str
     column_schema: str = Field(alias="schema")
@@ -65,10 +99,23 @@ class Column(PostgresNode):
     full_name: Optional[str] = None
 
     class Config:
+        """ """
+
         allow_population_by_field_name = True
 
     @validator("full_name", always=True)
     def make_full_name(cls, full_name, values):
+        """
+
+        Args:
+            full_name:
+            values:
+
+        Returns:
+
+        Raises:
+
+        """
         if full_name is not None:
             return full_name
         result = f"{values['column_schema']}.{values['table']}.{values['name']}"
@@ -76,12 +123,16 @@ class Column(PostgresNode):
 
 
 class Constraint(str, Enum):
+    """ """
+
     foreign_key = "f"
     primary_key = "p"
     belongs_to = "bt"
 
 
 class Edge(BaseModel):
+    """ """
+
     source: Union[ColumnID, TableID]
     destination: Union[ColumnID, TableID]
     definition: Optional[str]
@@ -90,6 +141,8 @@ class Edge(BaseModel):
 
 
 class TableType(str, Enum):
+    """ """
+
     Table = "BASE TABLE"
     View = "VIEW"
     ForeignTable = "FOREIGN"
@@ -97,6 +150,8 @@ class TableType(str, Enum):
 
 
 class Table(PostgresNode):
+    """ """
+
     name: str = Field(alias="table_name")
     table_schema: str = Field(alias="schema")
     table_type: TableType
@@ -106,16 +161,30 @@ class Table(PostgresNode):
     full_name: Optional[str] = None
 
     class Config:
+        """ """
+
         allow_population_by_field_name = True
 
     @validator("full_name", always=True)
     def make_full_name(cls, full_name, values):
+        """
+
+        Args:
+            full_name:
+            values:
+
+        Returns:
+
+        Raises:
+
+        """
         if full_name is not None:
             return full_name
 
         return f"{values['table_schema']}.{values['name']}"
 
     def get_edges(self):
+        """ """
         return [
             Edge(
                 constraint_type=Constraint("bt"),
@@ -136,6 +205,8 @@ class Table(PostgresNode):
 
 
 class EdgeQuery(BaseModel):
+    """ """
+
     namespace: str
     constraint_name: str
     constraint_type: str
@@ -147,8 +218,19 @@ class EdgeQuery(BaseModel):
     foreign_columns: List[str]
     definition: str
 
-    def to_edge(self) -> Edge:
-        assert len(self.self_columns) == 1 and len(self.foreign_columns) == 1
+    def to_edge(self) -> Optional[Edge]:
+        """
+
+        Args:
+
+        Returns:
+
+        Raises:
+
+        """
+        if not len(self.self_columns) == 1 and len(self.foreign_columns) == 1:
+            return None
+
         destination = ColumnID(
             table_schema=self.self_schema,
             table_name=self.self_table,
